//*******************************************************************************
//
//  File name :       sdr_lvds_and_logic_top_testbench.v
//
//  Description :     This module is the testbench for whole transceiver and additional logic. 
// 
//                    
//  Date - revision : 5/31/2005
//
//  Author :          Markus Adhiwiyogo
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale  1 ns / 10 ps
module SDR_LVDS_AND_LOGIC_TOP_TESTBENCH;

parameter CLOCK_CYCLE = 2; //Works best if it's an integer
//some simulators have difficulties with mathematical operands of clock.
//This clock cycle is the suggested value for simulation purposes only.

reg RST;
reg clk, REFCLK;

wire [1:0] SYS_CLK, REFCLK200;
wire [15:0] DATOUTP, DATOUTN;
wire FINAL_BUS_ALIGNED;
wire [0:0] CLKOUTP, CLKOUTN;
wire [63:0] RXDATA;

sdr_lvds_and_logic_top U_SDR_LVDS_AND_LOGIC(
	.CLK(SYS_CLK), .CLKI(SYS_CLK),
	.RXCLKINP(CLKOUTP), .RXCLKINN(CLKOUTN),
	.RSTn(RST),
	.RXDATINP(DATOUTP),
	.RXDATINN(DATOUTN),
	.CLKOUTP(CLKOUTP), 
	.CLKOUTN(CLKOUTN), 
	.DATOUTP(DATOUTP), 
	.DATOUTN(DATOUTN),
	.REFCLK(REFCLK200),  
	.RXDATA(RXDATA),
	.FINAL_CHECK(FINAL_CHECK),
	.FINAL_BUS_ALIGNED(FINAL_BUS_ALIGNED)
	);

assign SYS_CLK[0] = clk;
assign SYS_CLK[1] = ~clk;

initial
begin
 clk = 1'b1;
 RST = 1'b1;
 REFCLK = 1'b1;
end	

 
always
 #(CLOCK_CYCLE/2.0) clk = ~clk;


//200 MHz Reference Clock for IDELAY 
always
 #2.5 REFCLK = ~REFCLK;
 
assign REFCLK200[0] = REFCLK;
assign REFCLK200[1] = ~REFCLK;
 
initial
begin
#(0.3*CLOCK_CYCLE) RST = 1'b0;
#(50*CLOCK_CYCLE) RST = 1'b1;
#(100*CLOCK_CYCLE) RST = 1'b1;
#20000
#20000
$finish;
end


endmodule
