//*******************************************************************************
//
//  File name :       sdr_lvds_and_logic_top.v
//
//  Description :     This module is the top level for whole transceiver and additional logic. 
// 
//                    
//  Date - revision : 9/14/2005
//
//  Author :          Markus Adhiwiyogo
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale  1 ns / 10 ps
module sdr_lvds_and_logic_top(CLK, CLKI, RXCLKINP, RXCLKINN, REFCLK, RSTn, RXDATINP, RXDATINN, CLKOUTP, CLKOUTN, DATOUTP, DATOUTN, RXDATA, FINAL_CHECK, FINAL_BUS_ALIGNED);

input	[1:0]	CLK, REFCLK, CLKI;
input	[0:0]	RXCLKINP, RXCLKINN;
input		RSTn;
input	[15:0]	RXDATINP, RXDATINN;

output	[0:0]	CLKOUTP, CLKOUTN;
output	[15:0]	DATOUTP, DATOUTN;
output	[63:0]	RXDATA;
output		FINAL_CHECK;
output		FINAL_BUS_ALIGNED;

wire		DATA_ALIGNED;
wire		PREFINAL_CHECK;
wire	[4:1]	CHECK;
wire	[4:1]	PRECHECK;
wire		RXCLKDIV, TXCLKDIV, LOCKED;
reg	[63:0]	DATA_GEN_OUT;
wire	[63:0]	DATA_IN;
wire	[3:0]	TRAINING_PATTERN;
wire	[63:0]	DATA_SOURCE_BUS;


//Others

wire		GCLK, GCLKI, GCLKDIV, GCLKDCM, RST, RSTDCM;
wire		IRST;
wire		SEND_CLOCK;

assign	RSTDCM = ~RSTn;



wire	CLK_IBUF_BUFG;

//Replace the IBUFGDS for non-LVDS clock input
IBUFGDS_LVDSEXT_25 LVDS_CLK_IN(.O(CLK_FOR_FABRIC), .I(CLK[0]), .IB(CLK[1]));
assign GCLKI = CLK_FOR_FABRIC;


/* Tx/Rx Block Instantiation */

SDR_LVDS_TX_RX U_SDR_LVDS(
		.REFCLKP(REFCLK[0]), .REFCLKN(REFCLK[1]), 
		.RST(RST), .RSTDCM(RSTDCM),
		.CLKI(CLKI), .FABRIC_CLOCK(GCLKDIV),
		.CLKINP(RXCLKINP), .CLKINN(RXCLKINN), 
		.DATAINP(RXDATINP), .DATAINN(RXDATINN), 
		.DATA_FROM_ISERDES(RXDATA),
		.RXCLKDIV(RXCLKDIV),
		.CLKOUTP(CLKOUTP), .CLKOUTN(CLKOUTN), 
		.DATAOUTP(DATOUTP), .DATAOUTN(DATOUTN), 
		.DATA_TO_OSERDES(DATA_IN),
		.TXCLKDIV(TXCLKDIV),
		.USE_BITSLIP(USE_BITSLIP), 
		.TRAINING_PATTERN(TRAINING_PATTERN), 
		.DATA_ALIGNED(DATA_ALIGNED),
		.SRST_CTRL(~LOCKED),
		.IRST(IRST),
		.SEND_CLOCK(SEND_CLOCK),
		.BUS_ALIGNED(BUS_ALIGNED),
		.LOCKED(LOCKED),
		.GCLKDIV(GCLKDIV)
		);


/* Optional Settings for Rx */

assign TRAINING_PATTERN = 4'b1011; // User may choose pattern depends on preference
assign USE_BITSLIP = 1'b1; // Turn on/off the BITSLIP alignment


FDR FIN_BUS_ALIGNED(.Q(FINAL_BUS_ALIGNED), .C(RXCLKDIV), .D(BUS_ALIGNED), .R(RST));


/* Data Source */
always@(posedge GCLKDIV)
case({BUS_ALIGNED, SEND_CLOCK})
 2'b00: DATA_GEN_OUT = 64'hBBBBBBBBBBBBBBBB;
 2'b01: DATA_GEN_OUT = 64'hAAAAAAAAAAAAAAAA;
 2'b10: DATA_GEN_OUT = DATA_SOURCE_BUS;
 2'b11: DATA_GEN_OUT = DATA_SOURCE_BUS;
 default: DATA_GEN_OUT = 64'h0000000000000000;
endcase

assign DATA_IN = DATA_GEN_OUT;

//Pattern Generator for Simulation/Demo
data_source U_DATA_SOURCE(.CLK(GCLKDIV), .RST(RST), .DATA(DATA_SOURCE_BUS));


/* FIFO to store data from DATA_SOURCE */

// FIFO Control Pins
wire		RDEN1, WREN1, AEMPTY1, AFULL1, FULL1;
wire		RDEN12, WREN12, AEMPTY12, AFULL12, FULL12;
wire	[63:0]	DI1;
wire	[63:0]	DO1;
wire	[63:0]	RXDATA1_GCLK;


assign RDEN1 = ((((~AEMPTY1) & FINAL_BUS_ALIGNED) | AFULL1 | FULL1) & LOCKED); //Was AEMPTY
assign RDEN12 = ((((~AEMPTY12) & FINAL_BUS_ALIGNED) | AFULL12 | FULL12) & LOCKED);

assign WREN1 = (FINAL_BUS_ALIGNED &(~AFULL1));
assign WREN12 = (FINAL_BUS_ALIGNED &(~AFULL12));

assign DI1 = DATA_GEN_OUT;

//FIFO itself

FIFO16 U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0(
	.ALMOSTEMPTY(AEMPTY1), 
	.ALMOSTFULL(AFULL1), 
	.DO(DO1[31:0]), 
	.DOP(), 
	.EMPTY(), 
	.FULL(FULL1), 
	.RDCOUNT(), 
	.RDERR(), 
	.WRCOUNT(), 
	.WRERR(), 
	.DI(DI1[31:0]), 
	.DIP(), 
	.RDCLK(GCLKDIV), 
	.RDEN(RDEN1),
	.RST(RST), 
	.WRCLK(GCLKDIV), 
	.WREN(WREN1)
	);
	
FIFO16 U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1(
	.ALMOSTEMPTY(AEMPTY12), 
	.ALMOSTFULL(AFULL12), 
	.DO(DO1[63:32]), 
	.DOP(), 
	.EMPTY(), 
	.FULL(FULL12), 
	.RDCOUNT(), 
	.RDERR(), 
	.WRCOUNT(), 
	.WRERR(), 
	.DI(DI1[63:32]), 
	.DIP(), 
	.RDCLK(GCLKDIV), 
	.RDEN(RDEN12),
	.RST(RST), 
	.WRCLK(GCLKDIV), 
	.WREN(WREN12)
	);

// synopsys translate_off
defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0.ALMOST_FULL_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0.ALMOST_EMPTY_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0.DATA_WIDTH = 36;

defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1.ALMOST_FULL_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1.ALMOST_EMPTY_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1.DATA_WIDTH = 36;
// synopsys translate_on

// synthesis attribute ALMOST_FULL_OFFSET of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0 is 12'h032;
// synthesis attribute ALMOST_EMPTY_OFFSET of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0 is 12'h032;
// synthesis attribute DATA_WIDTH of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE0 is 36;

// synthesis attribute ALMOST_FULL_OFFSET of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1 is 12'h032;
// synthesis attribute ALMOST_EMPTY_OFFSET of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1 is 12'h032;
// synthesis attribute DATA_WIDTH of U_SDR_LVDS_TOP_FIFO16_FOR_DATA_SOURCE1 is 36;


//This module is used to place align the output of the FIFOs in the Tx and Rx clocks.
//The number of delays here is used for simulation purposes.
//Adaptive number of delays must be implemented for hardware implementation.
registers_chains U_TRC(.Q(RXDATA1_GCLK), .CLK(GCLKDIV), .CE(RDEN1), .RST(RST), .D(DO1[63:0])); //Delays the DATA_SOURCE FIFO output to match recovered data FIFO



/* FIFO to go from RXCLKDIV to GCLK domain */
/* DCM for demo purposes */
/* DSP48 for demo purposes */


/* DCM used for Synchronous RST */
DCM_ADV DCM_TX(
	.CLK0(GCLKDCM),
	.CLK180(),
	.CLK270(),
	.CLK2X(),
	.CLK2X180(),
	.CLK90(),
	.CLKDV(GCLKDIVDCM),
	.CLKFX(),
	.CLKFX180(),
	.DO(),
     	.DRDY(),
	.LOCKED(LOCKED),
	.CLKFB(GCLK),
	.CLKIN(GCLKI),
	.DADDR(),
        .DCLK(),
        .DEN(),
        .DI(),
        .DWE(),
        .PSCLK(),
        .PSEN(),
        .PSINCDEC(),
	.RST(RSTDCM)
	);

// synopsys translate_off
defparam DCM_TX.CLKDV_DIVIDE = 2.0;
defparam DCM_TX.CLKIN_DIVIDE_BY_2 = "TRUE";
defparam DCM_TX.CLK_FEEDBACK = "1X";
defparam DCM_TX.DFS_FREQUENCY_MODE = "HIGH";
defparam DCM_TX.DLL_FREQUENCY_MODE = "HIGH";
defparam DCM_TX.FACTORY_JF = 16'hF0F0;
// synopsys translate_on

// synthesis attribute CLKDV_DIVIDE of DCM_TX is 2.0
// synthesis attribute CLKIN_DIVIDE_BY_2 of DCM_TX is TRUE
// synthesis attribute CLK_FEEDBACK of DCM_TX is 1X
// synthesis attribute DFS_FREQUENCY_MODE of DCM_TX is HIGH
// synthesis attribute DLL_FREQUENCY_MODE of DCM_TX is HIGH
// synthesis attribute FACTORY_JF of DCM_TX is 16'hF0F0


//This module ensures all components in GCLKDIV domain are resetted properly
rst_machine DCM_RST_MACHINE
 (
 .CLK_generic(GCLKDIV),
 .RST_stimulus(~LOCKED), //Need to be changed to ~LOCKED for hardware implementation
 .IRDY(1'b1),
 .DOMAIN_RST(RST)
 );

// synthesis attribute MAX_FANOUT of RST is 30;
// synthesis attribute maxdelay of RST is 1.3 ns; 

BUFG CLK_BUFG_GCLK(.O(GCLK), .I(GCLKDCM));
BUFG CLK_BUFG_GCLKDIV(.O(GCLKDIV), .I(GCLKDIVDCM));

//Set this to TXCLKDIV for demo purpose
//assign GCLKDIV = TXCLKDIV;


/* Logic for Testing */


//Checking received data with data transmitted
wire	[47:0]	DSP48_OUTPUT, DSP48_OUTPUT2, DSP48_OUTPUT3, DSP48_OUTPUT4;

assign	PRECHECK[1] = (~(|(DSP48_OUTPUT)));
assign	PRECHECK[2] = (~(|(DSP48_OUTPUT2)));
assign	PRECHECK[3] = (~(|(DSP48_OUTPUT3)));
assign	PRECHECK[4] = (~(|(DSP48_OUTPUT4)));

FDR CHECK_REG1(.Q(CHECK[1]), .C(GCLKDIV), .D(PRECHECK[1]), .R(RST));
FDR CHECK_REG2(.Q(CHECK[2]), .C(GCLKDIV), .D(PRECHECK[2]), .R(RST));
FDR CHECK_REG3(.Q(CHECK[3]), .C(GCLKDIV), .D(PRECHECK[3]), .R(RST));
FDR CHECK_REG4(.Q(CHECK[4]), .C(GCLKDIV), .D(PRECHECK[4]), .R(RST));

assign PREFINAL_CHECK = &(CHECK[4:1]);

FDR FINAL_CHECK_REG(.Q(FINAL_CHECK), .C(GCLKDIV), .D(PREFINAL_CHECK), .R(RST));

//DSP48 to do wide logic comparator.
DSP48 U1
  (
    .A(18'h00000),        // Input A to Multiplier
    .B({1'b0,RXDATA[16:0]}),         // Input B to Multiplier
    .C({31'h00000000,RXDATA1_GCLK[16:0]}),  // Input C to Adder
    .BCIN(18'h00000),          //
    .PCIN(48'h000000000000),   //
    .OPMODE(7'b0110011),   //
    .SUBTRACT(1'b1),    //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(GCLKDIV),         //
    .CEA(1'b1),            //
    .CEB(1'b1),            //
    .CEC(1'b1),            //
    .CEP(1'b1),            //
    .CEM(1'b1),            //
    .CECTRL(1'b1),         //
    .CECARRYIN(1'b1),      //
    .CECINSUB(1'b1),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),              //
    .P(DSP48_OUTPUT),      //
    .PCOUT()               //
  );

DSP48 U2
  (
    .A(18'h00000),        // Input A to Multiplier
    .B({1'b0,RXDATA[33:17]}),         // Input B to Multiplier
    .C({31'h00000000,RXDATA1_GCLK[33:17]}),  // Input C to Adder
    .BCIN(18'h00000),          //
    .PCIN(48'h000000000000),   //
    .OPMODE(7'b0110011),   //
    .SUBTRACT(1'b1),    //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(GCLKDIV),         //
    .CEA(1'b1),            //
    .CEB(1'b1),            //
    .CEC(1'b1),            //
    .CEP(1'b1),            //
    .CEM(1'b1),            //
    .CECTRL(1'b1),         //
    .CECARRYIN(1'b1),      //
    .CECINSUB(1'b1),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),              //
    .P(DSP48_OUTPUT2),      //
    .PCOUT()               //
  );

DSP48 U3
  (
    .A(18'h00000),        // Input A to Multiplier
    .B({1'b0,RXDATA[50:34]}),         // Input B to Multiplier
    .C({31'h00000000,RXDATA1_GCLK[50:34]}),  // Input C to Adder
    .BCIN(18'h00000),          //
    .PCIN(48'h000000000000),   //
    .OPMODE(7'b0110011),   //
    .SUBTRACT(1'b1),    //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(GCLKDIV),         //
    .CEA(1'b1),            //
    .CEB(1'b1),            //
    .CEC(1'b1),            //
    .CEP(1'b1),            //
    .CEM(1'b1),            //
    .CECTRL(1'b1),         //
    .CECARRYIN(1'b1),      //
    .CECINSUB(1'b1),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),              //
    .P(DSP48_OUTPUT3),      //
    .PCOUT()               //
  );
  
DSP48 U4
  (
    .A(18'h00000),        // Input A to Multiplier
    .B({5'h00,RXDATA[63:51]}),         // Input B to Multiplier
    .C({35'h000000000,RXDATA1_GCLK[63:51]}),  // Input C to Adder
    .BCIN(18'h00000),          //
    .PCIN(48'h000000000000),   //
    .OPMODE(7'b0110011),   //
    .SUBTRACT(1'b1),    //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(GCLKDIV),         //
    .CEA(1'b1),            //
    .CEB(1'b1),            //
    .CEC(1'b1),            //
    .CEP(1'b1),            //
    .CEM(1'b1),            //
    .CECTRL(1'b1),         //
    .CECARRYIN(1'b1),      //
    .CECINSUB(1'b1),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),              //
    .P(DSP48_OUTPUT4),      //
    .PCOUT()               //
  );


//synthesis translate_off 
defparam U1.AREG = 2'b01;
defparam U1.BREG = 2'b01;
defparam U1.CARRYINREG = 1'b0;
defparam U1.MREG = 1'b1;
defparam U1.PREG = 1'b1;
defparam U1.OPMODEREG = 1'b0;
defparam U1.SUBTRACTREG = 1'b0;
defparam U1.CARRYINSELREG = 1'b0;
defparam U1.B_INPUT = "DIRECT";

defparam U2.AREG = 2'b01;
defparam U2.BREG = 2'b01;
defparam U2.CARRYINREG = 1'b0;
defparam U2.MREG = 1'b1;
defparam U2.PREG = 1'b1;
defparam U2.OPMODEREG = 1'b0;
defparam U2.SUBTRACTREG = 1'b0;
defparam U2.CARRYINSELREG = 1'b0;
defparam U2.B_INPUT = "DIRECT";

defparam U3.AREG = 2'b01;
defparam U3.BREG = 2'b01;
defparam U3.CARRYINREG = 1'b0;
defparam U3.MREG = 1'b1;
defparam U3.PREG = 1'b1;
defparam U3.OPMODEREG = 1'b0;
defparam U3.SUBTRACTREG = 1'b0;
defparam U3.CARRYINSELREG = 1'b0;
defparam U3.B_INPUT = "DIRECT";

defparam U4.AREG = 2'b01;
defparam U4.BREG = 2'b01;
defparam U4.CARRYINREG = 1'b0;
defparam U4.MREG = 1'b1;
defparam U4.PREG = 1'b1;
defparam U4.OPMODEREG = 1'b0;
defparam U4.SUBTRACTREG = 1'b0;
defparam U4.CARRYINSELREG = 1'b0;
defparam U4.B_INPUT = "DIRECT";
//synthesis translate_on

//synthesis attribute AREG of U1 is "1";
//synthesis attribute BREG of U1 is "1";
//synthesis attribute CARRYINREG of U1 is "0";
//synthesis attribute MREG of U1 is "1";
//synthesis attribute PREG of U1 is "1";
//synthesis attribute OPMODEREG of U1 is "0";
//synthesis attribute SUBTRACTREG of U1 is "0";
//synthesis attribute CARRYINSELREG of U1 is "0";
//synthesis attribute B_INPUT of U1 is "DIRECT";

//synthesis attribute AREG of U2 is "1";
//synthesis attribute BREG of U2 is "1";
//synthesis attribute CARRYINREG of U2 is "0";
//synthesis attribute MREG of U2 is "1";
//synthesis attribute PREG of U2 is "1";
//synthesis attribute OPMODEREG of U2 is "0";
//synthesis attribute SUBTRACTREG of U2 is "0";
//synthesis attribute CARRYINSELREG of U2 is "0";
//synthesis attribute B_INPUT of U2 is "DIRECT";

//synthesis attribute AREG of U3 is "1";
//synthesis attribute BREG of U3 is "1";
//synthesis attribute CARRYINREG of U3 is "0";
//synthesis attribute MREG of U3 is "1";
//synthesis attribute PREG of U3 is "1";
//synthesis attribute OPMODEREG of U3 is "0";
//synthesis attribute SUBTRACTREG of U3 is "0";
//synthesis attribute CARRYINSELREG of U3 is "0";
//synthesis attribute B_INPUT of U3 is "DIRECT";

//synthesis attribute AREG of U4 is "1";
//synthesis attribute BREG of U4 is "1";
//synthesis attribute CARRYINREG of U4 is "0";
//synthesis attribute MREG of U4 is "1";
//synthesis attribute PREG of U4 is "1";
//synthesis attribute OPMODEREG of U4 is "0";
//synthesis attribute SUBTRACTREG of U4 is "0";
//synthesis attribute CARRYINSELREG of U4 is "0";
//synthesis attribute B_INPUT of U4 is "DIRECT";


endmodule


`timescale  1 ns / 10 ps
module data_source(CLK, RST, DATA);

input         CLK, RST;
output [63:0] DATA;

reg [63:0] DATA;

wire bit0, bit16, bit32, bit48;

assign bit0  = ~(DATA[15+0 ] ^ DATA[14+0 ] ^ DATA[12+0 ] ^ DATA[3+0 ]);
assign bit16 = ~(DATA[15+16] ^ DATA[14+16] ^ DATA[12+16] ^ DATA[3+16]);
assign bit32 = ~(DATA[15+32] ^ DATA[14+32] ^ DATA[12+32] ^ DATA[3+32]);
assign bit48 = ~(DATA[15+48] ^ DATA[14+48] ^ DATA[12+48] ^ DATA[3+48]);

always @ (posedge CLK or posedge RST)
 begin
  if( RST == 1'b1)
      	DATA = 64'hDEAD_BEEF_600D_F00D;
  else
	DATA = {DATA[62:48],bit48,DATA[46:32],bit32,DATA[30:16],bit16,DATA[14:0],bit0};
 end

endmodule


`timescale  1 ns / 10 ps
module registers_chains(Q, CLK, CE, RST, D);

//This module is used to place align the output of the FIFOs in the Tx and Rx clocks.
//The number of delays here is used for simulation purposes.
//Adaptive number of delays must be implemented for hardware implementation.

input		CLK, CE, RST;
input	[63:0]	D;

output	[63:0]	Q;

wire	[63:0]	S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11, S12, S13;

n_bit_register reg1(.Q(S1), .CLK(CLK), .CE(CE), .RST(RST), .D(D));
n_bit_register reg2(.Q(S2), .CLK(CLK), .CE(CE), .RST(RST), .D(S1));
n_bit_register reg3(.Q(S3), .CLK(CLK), .CE(CE), .RST(RST), .D(S2));
n_bit_register reg4(.Q(S4), .CLK(CLK), .CE(CE), .RST(RST), .D(S3));
n_bit_register reg5(.Q(S5), .CLK(CLK), .CE(CE), .RST(RST), .D(S4));
n_bit_register reg6(.Q(S6), .CLK(CLK), .CE(CE), .RST(RST), .D(S5));
n_bit_register reg7(.Q(S7), .CLK(CLK), .CE(CE), .RST(RST), .D(S6));
n_bit_register reg8(.Q(S8), .CLK(CLK), .CE(CE), .RST(RST), .D(S7));
n_bit_register reg9(.Q(S9), .CLK(CLK), .CE(CE), .RST(RST), .D(S8));
n_bit_register reg10(.Q(S10), .CLK(CLK), .CE(CE), .RST(RST), .D(S9));
n_bit_register reg11(.Q(S11), .CLK(CLK), .CE(CE), .RST(RST), .D(S10));
n_bit_register reg12(.Q(S12), .CLK(CLK), .CE(CE), .RST(RST), .D(S11));
n_bit_register reg13(.Q(S13), .CLK(CLK), .CE(CE), .RST(RST), .D(S12));
n_bit_register reg14(.Q(Q), .CLK(CLK), .CE(CE), .RST(RST), .D(S13));

endmodule


`timescale  1 ns / 10 ps
module n_bit_register(Q, CLK, CE, RST, D);

input		CLK, CE, RST;
input	[63:0]	D;

output [63:0]	Q;

FDRE bit00(.Q(Q[0]), .C(CLK), .CE(CE), .D(D[0]), .R(RST));
FDRE bit01(.Q(Q[1]), .C(CLK), .CE(CE), .D(D[1]), .R(RST));
FDRE bit02(.Q(Q[2]), .C(CLK), .CE(CE), .D(D[2]), .R(RST));
FDRE bit03(.Q(Q[3]), .C(CLK), .CE(CE), .D(D[3]), .R(RST));
FDRE bit04(.Q(Q[4]), .C(CLK), .CE(CE), .D(D[4]), .R(RST));
FDRE bit05(.Q(Q[5]), .C(CLK), .CE(CE), .D(D[5]), .R(RST));
FDRE bit06(.Q(Q[6]), .C(CLK), .CE(CE), .D(D[6]), .R(RST));
FDRE bit07(.Q(Q[7]), .C(CLK), .CE(CE), .D(D[7]), .R(RST));
FDRE bit08(.Q(Q[8]), .C(CLK), .CE(CE), .D(D[8]), .R(RST));
FDRE bit09(.Q(Q[9]), .C(CLK), .CE(CE), .D(D[9]), .R(RST));
FDRE bit10(.Q(Q[10]), .C(CLK), .CE(CE), .D(D[10]), .R(RST));
FDRE bit11(.Q(Q[11]), .C(CLK), .CE(CE), .D(D[11]), .R(RST));
FDRE bit12(.Q(Q[12]), .C(CLK), .CE(CE), .D(D[12]), .R(RST));
FDRE bit13(.Q(Q[13]), .C(CLK), .CE(CE), .D(D[13]), .R(RST));
FDRE bit14(.Q(Q[14]), .C(CLK), .CE(CE), .D(D[14]), .R(RST));
FDRE bit15(.Q(Q[15]), .C(CLK), .CE(CE), .D(D[15]), .R(RST));
FDRE bit16(.Q(Q[16]), .C(CLK), .CE(CE), .D(D[16]), .R(RST));
FDRE bit17(.Q(Q[17]), .C(CLK), .CE(CE), .D(D[17]), .R(RST));
FDRE bit18(.Q(Q[18]), .C(CLK), .CE(CE), .D(D[18]), .R(RST));
FDRE bit19(.Q(Q[19]), .C(CLK), .CE(CE), .D(D[19]), .R(RST));
FDRE bit20(.Q(Q[20]), .C(CLK), .CE(CE), .D(D[20]), .R(RST));
FDRE bit21(.Q(Q[21]), .C(CLK), .CE(CE), .D(D[21]), .R(RST));
FDRE bit22(.Q(Q[22]), .C(CLK), .CE(CE), .D(D[22]), .R(RST));
FDRE bit23(.Q(Q[23]), .C(CLK), .CE(CE), .D(D[23]), .R(RST));
FDRE bit24(.Q(Q[24]), .C(CLK), .CE(CE), .D(D[24]), .R(RST));
FDRE bit25(.Q(Q[25]), .C(CLK), .CE(CE), .D(D[25]), .R(RST));
FDRE bit26(.Q(Q[26]), .C(CLK), .CE(CE), .D(D[26]), .R(RST));
FDRE bit27(.Q(Q[27]), .C(CLK), .CE(CE), .D(D[27]), .R(RST));
FDRE bit28(.Q(Q[28]), .C(CLK), .CE(CE), .D(D[28]), .R(RST));
FDRE bit29(.Q(Q[29]), .C(CLK), .CE(CE), .D(D[29]), .R(RST));
FDRE bit30(.Q(Q[30]), .C(CLK), .CE(CE), .D(D[30]), .R(RST));
FDRE bit31(.Q(Q[31]), .C(CLK), .CE(CE), .D(D[31]), .R(RST));
FDRE bit32(.Q(Q[32]), .C(CLK), .CE(CE), .D(D[32]), .R(RST));
FDRE bit33(.Q(Q[33]), .C(CLK), .CE(CE), .D(D[33]), .R(RST));
FDRE bit34(.Q(Q[34]), .C(CLK), .CE(CE), .D(D[34]), .R(RST));
FDRE bit35(.Q(Q[35]), .C(CLK), .CE(CE), .D(D[35]), .R(RST));
FDRE bit36(.Q(Q[36]), .C(CLK), .CE(CE), .D(D[36]), .R(RST));
FDRE bit37(.Q(Q[37]), .C(CLK), .CE(CE), .D(D[37]), .R(RST));
FDRE bit38(.Q(Q[38]), .C(CLK), .CE(CE), .D(D[38]), .R(RST));
FDRE bit39(.Q(Q[39]), .C(CLK), .CE(CE), .D(D[39]), .R(RST));
FDRE bit40(.Q(Q[40]), .C(CLK), .CE(CE), .D(D[40]), .R(RST));
FDRE bit41(.Q(Q[41]), .C(CLK), .CE(CE), .D(D[41]), .R(RST));
FDRE bit42(.Q(Q[42]), .C(CLK), .CE(CE), .D(D[42]), .R(RST));
FDRE bit43(.Q(Q[43]), .C(CLK), .CE(CE), .D(D[43]), .R(RST));
FDRE bit44(.Q(Q[44]), .C(CLK), .CE(CE), .D(D[44]), .R(RST));
FDRE bit45(.Q(Q[45]), .C(CLK), .CE(CE), .D(D[45]), .R(RST));
FDRE bit46(.Q(Q[46]), .C(CLK), .CE(CE), .D(D[46]), .R(RST));
FDRE bit47(.Q(Q[47]), .C(CLK), .CE(CE), .D(D[47]), .R(RST));
FDRE bit48(.Q(Q[48]), .C(CLK), .CE(CE), .D(D[48]), .R(RST));
FDRE bit49(.Q(Q[49]), .C(CLK), .CE(CE), .D(D[49]), .R(RST));
FDRE bit50(.Q(Q[50]), .C(CLK), .CE(CE), .D(D[50]), .R(RST));
FDRE bit51(.Q(Q[51]), .C(CLK), .CE(CE), .D(D[51]), .R(RST));
FDRE bit52(.Q(Q[52]), .C(CLK), .CE(CE), .D(D[52]), .R(RST));
FDRE bit53(.Q(Q[53]), .C(CLK), .CE(CE), .D(D[53]), .R(RST));
FDRE bit54(.Q(Q[54]), .C(CLK), .CE(CE), .D(D[54]), .R(RST));
FDRE bit55(.Q(Q[55]), .C(CLK), .CE(CE), .D(D[55]), .R(RST));
FDRE bit56(.Q(Q[56]), .C(CLK), .CE(CE), .D(D[56]), .R(RST));
FDRE bit57(.Q(Q[57]), .C(CLK), .CE(CE), .D(D[57]), .R(RST));
FDRE bit58(.Q(Q[58]), .C(CLK), .CE(CE), .D(D[58]), .R(RST));
FDRE bit59(.Q(Q[59]), .C(CLK), .CE(CE), .D(D[59]), .R(RST));
FDRE bit60(.Q(Q[60]), .C(CLK), .CE(CE), .D(D[60]), .R(RST));
FDRE bit61(.Q(Q[61]), .C(CLK), .CE(CE), .D(D[61]), .R(RST));
FDRE bit62(.Q(Q[62]), .C(CLK), .CE(CE), .D(D[62]), .R(RST));
FDRE bit63(.Q(Q[63]), .C(CLK), .CE(CE), .D(D[63]), .R(RST));

endmodule


