//*******************************************************************************
//
//  File name :       sdr_lvds_tx_rx.v
//
//  Description :     This module is the top level for Tx/Rx only. 
// 
//                    
//  Date - revision : 9/14/2005
//
//  Author :          Markus Adhiwiyogo
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************


`timescale  1 ns / 10 ps
module SDR_LVDS_TX_RX(

		// System Pins
		REFCLKP, REFCLKN, 
		RST, RSTDCM,
		CLKI,
		FABRIC_CLOCK,
		
		//Rx Pins
		CLKINP, CLKINN, 
		DATAINP, DATAINN, 
		DATA_FROM_ISERDES,
		RXCLKDIV,
		
		//Tx Pins
		CLKOUTP, CLKOUTN, 
		DATAOUTP, DATAOUTN, 
		DATA_TO_OSERDES,
		TXCLKDIV,
		
		//Rx Control Pins
		DATA_ALIGNED,

		SRST_CTRL,
		IRST,
		SEND_CLOCK,
		BUS_ALIGNED,
		
		LOCKED,
		GCLKDIV
		);

/* IO Pins */

// System Pins
input		REFCLKP, REFCLKN;	//IDELAY Reference Clock
input		RST, RSTDCM;		//FPGA System Reset, DCM System Reset
input	[1:0]	CLKI;			//FPGA Fast Clock for TX (0 is P, 1 is N)
input		FABRIC_CLOCK;		//Already in GCLK

//Rx Pins
input	[0:0]	CLKINP, CLKINN;		//Rx Clock Input (external to FPGA)
input	[15:0]	DATAINP, DATAINN;	//Rx Data Input	(external to FPGA)
output	[63:0]	DATA_FROM_ISERDES;	//Recovered Rx Data at RXCLKDIV domain
output		RXCLKDIV;		//RXCLKDIV domain

//Tx Pins
output	[0:0]	CLKOUTP, CLKOUTN;	//Tx Clock Output (FPGA to external)
output	[15:0]	DATAOUTP, DATAOUTN;	//Tx Data Output (FPGA to external)
input	[63:0]	DATA_TO_OSERDES;	//Data at TXCLKDIV domain to be sent as Tx Data

//Rx Control Pins
output		DATA_ALIGNED;		//If pin is HIGH, design has finished data alignment with IDELAY and BITLSIP
output		TXCLKDIV;		//TXCLKDIV domain
output		BUS_ALIGNED;

input		SRST_CTRL;
output		IRST;
output		SEND_CLOCK;

input		GCLKDIV;
input		LOCKED;

wire		TXCLK;			//Tx Clock Wire
wire		REFCLKPAD, REFCLK200;	//REFCLK Wires/Interconnects
wire		SRST, SCE, ORST;		//SERDES Reset and Clock Enable
wire		IDLY_RST;
wire	[2:0]	IRDY_BUS;
//wire		TXDCMLOCKED;

/* 200MHz Reference Clock Input for IDELAYCTRL */

IBUFGDS_LVDSEXT_25 REFCLK_IN(.O(REFCLKPAD), .I(REFCLKP), .IB(REFCLKN));
BUFG REFCLK_BUFG(.O(REFCLK200), .I(REFCLKPAD));
IDELAYCTRL RX_IDELAYCTRL1(.RDY(IRDY_BUS[0]), .REFCLK(REFCLK200), .RST(IDLY_RST));
IDELAYCTRL RX_IDELAYCTRL2(.RDY(IRDY_BUS[1]), .REFCLK(REFCLK200), .RST(IDLY_RST));
IDELAYCTRL RX_IDELAYCTRL3(.RDY(IRDY_BUS[2]), .REFCLK(REFCLK200), .RST(IDLY_RST));
rst_machine IDELAYCTRL_RST_MACHINE(.CLK_generic(REFCLK200), .RST_stimulus(RST), .IRDY(1'b1), .DOMAIN_RST(IDLY_RST));

assign IRDY = &IRDY_BUS;

// synthesis attribute MAX_FANOUT of IDLY_RST is 5;
// synthesis attribute maxdelay of IDLY_RST is 1.3 ns; 

/* Reset Control */ 

//LOCKED set to 1'b1 if no DCM if used, otherwise set to LOCK pin of DCM

rst_machine RST_CTRL0(.CLK_generic(TXCLKDIV), .RST_stimulus(SRST_CTRL), .IRDY(1'b1), .DOMAIN_RST(ORST)); //Synchronous reset control for all blocks clocked by TXCLK and/or TXCLKDIV
rst_machine RST_CTRL1(.CLK_generic(RXCLKDIV), .RST_stimulus((~SRST_CTRL & ~IRDY)), .IRDY(IRDY), .DOMAIN_RST(IRST)); //Synchronous reset control for all blocks clocked by RXCLK and/or RXCLKDIV

// synthesis attribute MAX_FANOUT of ORST is 50;
// synthesis attribute maxdelay of ORST is 1.3 ns; 
// synthesis attribute MAX_FANOUT of IRST is 50; 
// synthesis attribute maxdelay of IRST is 1.3 ns; 

assign SCE = 1'b1;

/* Tx Blocks */

tx_clk_and_dat U_TX(.DATA_IN(DATA_TO_OSERDES), .CLKI(CLKI), .FABRIC_CLOCK(FABRIC_CLOCK), .ORST(ORST), .OCE(SCE), .CLKOUTP(CLKOUTP), .CLKOUTN(CLKOUTN), .DATOUTP(DATAOUTP), .DATOUTN(DATAOUTN), .TXCLK(TXCLK), .TXCLKDIV(TXCLKDIV));


/* Rx Block */

rx_clk_and_dat U_RX(.CLKINP(CLKINP), .CLKINN(CLKINN), .DATAINP(DATAINP), .DATAINN(DATAINN), .IRDY(IRDY), .USE_BITSLIP(1'b0), .TRAINING_PATTERN(4'b0000), .DATA_OUT(DATA_FROM_ISERDES), .RST(IRST), .IRST(IRST), .SCE(SCE), .RXCLKDIV(RXCLKDIV), .DATA_ALIGNED(DATA_ALIGNED), .SEND_CLOCK(SEND_CLOCK), .BUS_ALIGNED(BUS_ALIGNED), .LOCKED(LOCKED), .GCLKDIV(GCLKDIV));

endmodule


`timescale  1 ns / 10 ps
module tx_clk_and_dat (DATA_IN, CLKI, FABRIC_CLOCK, ORST,  OCE, CLKOUTP, CLKOUTN, DATOUTP, DATOUTN, TXCLK, TXCLKDIV);

input		ORST, OCE;
input	[63:0]	DATA_IN;
input	[1:0]	CLKI; //FPGA Fast Clock for Tx (0 is P, 1 is N)
input	[0:0]	FABRIC_CLOCK;

output 	[0:0] 	CLKOUTP, CLKOUTN;
output	[15:0]	DATOUTP, DATOUTN;
output		TXCLK, TXCLKDIV;

wire	[0:0]	PRECLKOUT;
wire	[15:0]	PREDATOUT;
wire		BUFIOIN;
wire		RDEN, WREN, RDEN2, WREN2, AEMPTY, AEMPTY2, AFULL, AFULL2, FULL, FULL2;
wire	[63:0]	DI;

//Data Input FIFO to move clock domain.
assign RDEN = ((~AEMPTY) | AFULL | FULL);
assign WREN = (~AFULL);

assign RDEN2 = ((~AEMPTY2) | AFULL2 | FULL2);
assign WREN2 = (~AFULL2);

//Data FIFO to move from RXCLKDIV domain to GCLKDIV domain
FIFO16 TX_CLK_AND_DAT_FIFO160(.ALMOSTEMPTY(AEMPTY), .ALMOSTFULL(AFULL), .DO(DI[31:0]), .DOP(), .EMPTY(), .FULL(FULL), .RDCOUNT(), .RDERR(), .WRCOUNT(), .WRERR(), .DI(DATA_IN[31:0]), .DIP(), .RDCLK(TXCLKDIV), .RDEN(RDEN), .RST(ORST), .WRCLK(FABRIC_CLOCK), .WREN(WREN));
FIFO16 TX_CLK_AND_DAT_FIFO161(.ALMOSTEMPTY(AEMPTY2), .ALMOSTFULL(AFULL2), .DO(DI[63:32]), .DOP(), .EMPTY(), .FULL(FULL2), .RDCOUNT(), .RDERR(), .WRCOUNT(), .WRERR(), .DI(DATA_IN[63:32]), .DIP(), .RDCLK(TXCLKDIV), .RDEN(RDEN2),.RST(ORST), .WRCLK(FABRIC_CLOCK), .WREN(WREN2));

// synopsys translate_off
defparam TX_CLK_AND_DAT_FIFO160.ALMOST_FULL_OFFSET = 12'h005;
defparam TX_CLK_AND_DAT_FIFO160.ALMOST_EMPTY_OFFSET = 12'h005;
defparam TX_CLK_AND_DAT_FIFO160.DATA_WIDTH = 36;

defparam TX_CLK_AND_DAT_FIFO161.ALMOST_FULL_OFFSET = 12'h005;
defparam TX_CLK_AND_DAT_FIFO161.ALMOST_EMPTY_OFFSET = 12'h005;
defparam TX_CLK_AND_DAT_FIFO161.DATA_WIDTH = 36;	
// synopsys translate_on

// synthesis attribute ALMOST_FULL_OFFSET of TX_CLK_AND_DAT_FIFO160 is 12'h005;
// synthesis attribute ALMOST_EMPTY_OFFSET of TX_CLK_AND_DAT_FIFO160 is 12'h005;
// synthesis attribute DATA_WIDTH of TX_CLK_AND_DAT_FIFO160 is 36;

// synthesis attribute ALMOST_FULL_OFFSET of TX_CLK_AND_DAT_FIFO161 is 12'h005;
// synthesis attribute ALMOST_EMPTY_OFFSET of TX_CLK_AND_DAT_FIFO161 is 12'h005;
// synthesis attribute DATA_WIDTH of TX_CLK_AND_DAT_FIFO161 is 36;


//Clock Generator Codes
IBUFGDS_LVDSEXT_25 CLKI_INPUT(.O(BUFIOIN), .I(CLKI[0]), .IB(CLKI[1]));
BUFIO TX_CLK_BUFIO(.O(TXCLK), .I(BUFIOIN));
BUFR TX_CLK_BUFR(.O(TXCLKDIV), .CE(1'b1), .CLR(1'b0), .I(TXCLK)); //BUFR may have its CLR pin connected to RST

//synopsys translate_off
defparam TX_CLK_BUFR.BUFR_DIVIDE = "4";
//synopsys translate_on

// synthesis attribute BUFR_DIVIDE of TX_CLK_BUFR is 4
// synthesis attribute buffer_type of TXCLKDIV is none


//ODDR Codes used for forwarding clock(s)
ODDR TX_CLK_OUT_00(.Q(PRECLKOUT[0]), .C(TXCLK), .CE(OCE), .D1(1'b1), .D2(1'b0), .R(1'b0), .S(1'b0));

// synopsys translate_off
defparam TX_CLK_OUT_00.SRTYPE = "ASYNC";
// synopsys translate_on

// synthesis attribute SRTYPE of TX_CLK_OUT_00 is ASYNC

OBUFDS_LVDSEXT_25 CLK_OLVDS_00(.O(CLKOUTP[0]), .OB(CLKOUTN[0]), .I(PRECLKOUT[0]));

//Tx data out
OSERDES TX_DAT_OUT_00(.OQ(PREDATOUT[0]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[0]), .D2(DI[1]), .D3(DI[2]), .D4(DI[3]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_01(.OQ(PREDATOUT[1]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[4]), .D2(DI[5]), .D3(DI[6]), .D4(DI[7]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_02(.OQ(PREDATOUT[2]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[8]), .D2(DI[9]), .D3(DI[10]), .D4(DI[11]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_03(.OQ(PREDATOUT[3]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[12]), .D2(DI[13]), .D3(DI[14]), .D4(DI[15]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_04(.OQ(PREDATOUT[4]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[16]), .D2(DI[17]), .D3(DI[18]), .D4(DI[19]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_05(.OQ(PREDATOUT[5]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[20]), .D2(DI[21]), .D3(DI[22]), .D4(DI[23]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_06(.OQ(PREDATOUT[6]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[24]), .D2(DI[25]), .D3(DI[26]), .D4(DI[27]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_07(.OQ(PREDATOUT[7]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[28]), .D2(DI[29]), .D3(DI[30]), .D4(DI[31]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_08(.OQ(PREDATOUT[8]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[32]), .D2(DI[33]), .D3(DI[34]), .D4(DI[35]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_09(.OQ(PREDATOUT[9]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[36]), .D2(DI[37]), .D3(DI[38]), .D4(DI[39]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_10(.OQ(PREDATOUT[10]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[40]), .D2(DI[41]), .D3(DI[42]), .D4(DI[43]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_11(.OQ(PREDATOUT[11]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[44]), .D2(DI[45]), .D3(DI[46]), .D4(DI[47]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_12(.OQ(PREDATOUT[12]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[48]), .D2(DI[49]), .D3(DI[50]), .D4(DI[51]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_13(.OQ(PREDATOUT[13]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[52]), .D2(DI[53]), .D3(DI[54]), .D4(DI[55]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_14(.OQ(PREDATOUT[14]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[56]), .D2(DI[57]), .D3(DI[58]), .D4(DI[59]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());
OSERDES TX_DAT_OUT_15(.OQ(PREDATOUT[15]), .SHIFTOUT1(), .SHIFTOUT2(), .TQ(), .CLK(TXCLK), .CLKDIV(TXCLKDIV), .D1(DI[60]), .D2(DI[61]), .D3(DI[62]), .D4(DI[63]), .D5(), .D6(), .OCE(OCE), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(ORST), .T1(), .T2(), .T3(), .T4(), .TCE());

// synopsys translate_off
defparam TX_DAT_OUT_00.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_01.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_02.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_03.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_04.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_05.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_06.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_07.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_08.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_09.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_10.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_11.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_12.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_13.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_14.DATA_RATE_OQ = "SDR";
defparam TX_DAT_OUT_15.DATA_RATE_OQ = "SDR";

defparam TX_DAT_OUT_00.DATA_WIDTH = 4;
defparam TX_DAT_OUT_01.DATA_WIDTH = 4;
defparam TX_DAT_OUT_02.DATA_WIDTH = 4;
defparam TX_DAT_OUT_03.DATA_WIDTH = 4;
defparam TX_DAT_OUT_04.DATA_WIDTH = 4;
defparam TX_DAT_OUT_05.DATA_WIDTH = 4;
defparam TX_DAT_OUT_06.DATA_WIDTH = 4;
defparam TX_DAT_OUT_07.DATA_WIDTH = 4;
defparam TX_DAT_OUT_08.DATA_WIDTH = 4;
defparam TX_DAT_OUT_09.DATA_WIDTH = 4;
defparam TX_DAT_OUT_10.DATA_WIDTH = 4;
defparam TX_DAT_OUT_11.DATA_WIDTH = 4;
defparam TX_DAT_OUT_12.DATA_WIDTH = 4;
defparam TX_DAT_OUT_13.DATA_WIDTH = 4;
defparam TX_DAT_OUT_14.DATA_WIDTH = 4;
defparam TX_DAT_OUT_15.DATA_WIDTH = 4;

defparam TX_DAT_OUT_00.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_01.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_02.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_03.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_04.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_05.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_06.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_07.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_08.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_09.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_10.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_11.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_12.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_13.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_14.SERDES_MODE = "MASTER";
defparam TX_DAT_OUT_15.SERDES_MODE = "MASTER";
// synopsys translate_on

// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_00 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_01 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_02 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_03 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_04 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_05 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_06 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_07 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_08 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_09 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_10 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_11 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_12 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_13 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_14 is SDR
// synthesis attribute DATA_RATE_OQ of TX_DAT_OUT_15 is SDR

// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_00 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_01 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_02 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_03 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_04 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_05 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_06 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_07 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_08 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_09 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_10 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_11 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_12 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_13 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_14 is BUF
// synthesis attribute DATA_RATE_TQ of TX_DAT_OUT_15 is BUF

// synthesis attribute DATA_WIDTH of TX_DAT_OUT_00 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_01 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_02 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_03 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_04 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_05 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_06 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_07 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_08 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_09 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_10 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_11 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_12 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_13 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_14 is 4
// synthesis attribute DATA_WIDTH of TX_DAT_OUT_15 is 4

// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_00 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_01 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_02 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_03 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_04 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_05 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_06 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_07 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_08 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_09 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_10 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_11 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_12 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_13 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_14 is 1
// synthesis attribute TRISTATE_WIDTH of TX_DAT_OUT_15 is 1

// synthesis attribute SERDES_MODE of TX_DAT_OUT_00 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_01 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_02 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_03 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_04 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_05 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_06 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_07 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_08 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_09 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_10 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_11 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_12 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_13 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_14 is MASTER
// synthesis attribute SERDES_MODE of TX_DAT_OUT_15 is MASTER

OBUFDS_LVDSEXT_25 DAT_OLVDS_00(.O(DATOUTP[0]), .OB(DATOUTN[0]), .I(PREDATOUT[0]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_01(.O(DATOUTP[1]), .OB(DATOUTN[1]), .I(PREDATOUT[1]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_02(.O(DATOUTP[2]), .OB(DATOUTN[2]), .I(PREDATOUT[2]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_03(.O(DATOUTP[3]), .OB(DATOUTN[3]), .I(PREDATOUT[3]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_04(.O(DATOUTP[4]), .OB(DATOUTN[4]), .I(PREDATOUT[4]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_05(.O(DATOUTP[5]), .OB(DATOUTN[5]), .I(PREDATOUT[5]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_06(.O(DATOUTP[6]), .OB(DATOUTN[6]), .I(PREDATOUT[6]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_07(.O(DATOUTP[7]), .OB(DATOUTN[7]), .I(PREDATOUT[7]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_08(.O(DATOUTP[8]), .OB(DATOUTN[8]), .I(PREDATOUT[8]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_09(.O(DATOUTP[9]), .OB(DATOUTN[9]), .I(PREDATOUT[9]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_10(.O(DATOUTP[10]), .OB(DATOUTN[10]), .I(PREDATOUT[10]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_11(.O(DATOUTP[11]), .OB(DATOUTN[11]), .I(PREDATOUT[11]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_12(.O(DATOUTP[12]), .OB(DATOUTN[12]), .I(PREDATOUT[12]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_13(.O(DATOUTP[13]), .OB(DATOUTN[13]), .I(PREDATOUT[13]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_14(.O(DATOUTP[14]), .OB(DATOUTN[14]), .I(PREDATOUT[14]));
OBUFDS_LVDSEXT_25 DAT_OLVDS_15(.O(DATOUTP[15]), .OB(DATOUTN[15]), .I(PREDATOUT[15]));


endmodule


`timescale  1 ns / 10 ps
module rx_clk_and_dat (CLKINP, CLKINN, DATAINP, DATAINN, IRDY, USE_BITSLIP, TRAINING_PATTERN, DATA_OUT, RST, IRST, SCE, RXCLKDIV, DATA_ALIGNED, SEND_CLOCK, BUS_ALIGNED, LOCKED, GCLKDIV);

input	[0:0]	CLKINP, CLKINN;
input	[15:0]	DATAINP, DATAINN;
input		IRDY, USE_BITSLIP;
output	[63:0]	DATA_OUT;
input		RST, IRST, SCE;
input	[3:0]	TRAINING_PATTERN;
input		LOCKED;
input		GCLKDIV;

output		RXCLKDIV;
output		DATA_ALIGNED;
output		BUS_ALIGNED;
output		SEND_CLOCK;

//Control and Data Pins
wire	[0:0]	CLKPAD;
wire		RXCLK;
wire	[15:0]	DATAPAD;
wire	[15:0]	IDELAY2FDR; //Wires to monitor how the sampled data move when using IDELAY

//reg	[15:0]	BITSLIP, ICE, INC;
//reg		SAMPLED_CLOCK;

wire		BITSLIP, ICE, INC;

// FIFO Control Pins
wire	[63:0]	DI;
wire		FINAL_BUS_ALIGNED;
wire		AEMPTY, AFULL, RDEN, FULL;
wire		AEMPTY2, AFULL2, RDEN2, FULL2;



/* Receive Clock Portion */

wire	[3:0]	SAMPLED_CLOCK;
wire		BUFIOIN;

wire	[3:0]	SAMPLED_CLOCKx;
wire		BUFIOINx;


IBUFDS_LVDSEXT_25 LVDS_RX_CLK_IN0(.O(CLKPAD[0]), .I(CLKINP[0]), .IB(CLKINN[0]));
ISERDES ISERDES_CLK_IN00(.O(BUFIOIN), .Q1(SAMPLED_CLOCK[3]), .Q2(SAMPLED_CLOCK[2]), .Q3(SAMPLED_CLOCK[1]), .Q4(SAMPLED_CLOCK[0]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(), .CE1(1'b1), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(CLKPAD[0]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));

// synopsys translate_off
defparam ISERDES_CLK_IN00.BITSLIP_ENABLE = "FALSE";
defparam ISERDES_CLK_IN00.DATA_RATE = "SDR";
defparam ISERDES_CLK_IN00.DATA_WIDTH = 4;
defparam ISERDES_CLK_IN00.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CLK_IN00.IOBDELAY = "IFD";
defparam ISERDES_CLK_IN00.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CLK_IN00.NUM_CE = 1;
defparam ISERDES_CLK_IN00.SERDES_MODE = "MASTER";
// synopsys translate_on

// synthesis attribute BITSLIP_ENABLE of ISERDES_CLK_IN00 is FALSE;
// synthesis attribute DATA_RATE of ISERDES_CLK_IN00 is SDR;
// synthesis attribute DATA_WIDTH of ISERDES_CLK_IN00 is 4;
// synthesis attribute INTERFACE_TYPE of ISERDES_CLK_IN00 is NETWORKING;
// synthesis attribute IOBDELAY of ISERDES_CLK_IN00 is IFD;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CLK_IN00 is VARIABLE;
// synthesis attribute NUM_CE of ISERDES_CLK_IN00 is 1;
// synthesis attribute SERDES_MODE of ISERDES_CLK_IN00 is MASTER;

/*
ISERDES ISERDES_CLK_IN00x(.O(BUFIOINx), .Q1(SAMPLED_CLOCKx[3]), .Q2(SAMPLED_CLOCKx[2]), .Q3(SAMPLED_CLOCKx[1]), .Q4(SAMPLED_CLOCKx[0]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(), .CE1(1'b1), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(CLKPAD[0]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));

// synopsys translate_off
defparam ISERDES_CLK_IN00x.BITSLIP_ENABLE = "FALSE";
defparam ISERDES_CLK_IN00x.DATA_RATE = "SDR";
defparam ISERDES_CLK_IN00x.DATA_WIDTH = 4;
defparam ISERDES_CLK_IN00x.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CLK_IN00x.IOBDELAY = "BOTH";
defparam ISERDES_CLK_IN00x.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CLK_IN00x.NUM_CE = 1;
defparam ISERDES_CLK_IN00x.SERDES_MODE = "MASTER";
// synopsys translate_on
*/    
BUFIO RX_CLK_BUFIO(.O(RXCLK), .I(BUFIOIN));
BUFR RX_CLK_BUFR(.O(RXCLKDIV), .CE(1'b1), .CLR(1'b0), .I(RXCLK)); //BUFR may have its CLR pin connected to RST

//synopsys translate_off
defparam RX_CLK_BUFR.BUFR_DIVIDE = "4";
//synopsys translate_on

// synthesis attribute BUFR_DIVIDE of RX_CLK_BUFR is 4
// synthesis attribute buffer_type of RXCLKDIV is none

assign BUS_ALIGNED = DATA_ALIGNED;

wire	[5:0]	RST_extend;

FD Extend_Resets0(.Q(RST_extend[0]), .C(RXCLKDIV), .D(RST));
FD Extend_Resets1(.Q(RST_extend[1]), .C(RXCLKDIV), .D(RST_extend[0]));
FD Extend_Resets2(.Q(RST_extend[2]), .C(RXCLKDIV), .D(RST_extend[1]));
FD Extend_Resets3(.Q(RST_extend[3]), .C(RXCLKDIV), .D(RST_extend[2]));
FD Extend_Resets4(.Q(RST_extend[4]), .C(RXCLKDIV), .D(RST_extend[3]));
FD Extend_Resets5(.Q(RST_extend[5]), .C(RXCLKDIV), .D(RST_extend[4]));
iserdes_alignment_machine U_ALIGN00(.RXCLKDIV(RXCLKDIV), .RST(RST_extend[5]), .SAMPLED_CLOCK(SAMPLED_CLOCK), .IRDY(IRDY), .USE_BITSLIP(USE_BITSLIP), .RXDATA(DI[3:0]), .TRAINING_PATTERN(TRAINING_PATTERN), .INC(INC), .ICE(ICE), .BITSLIP(BITSLIP), .DATA_ALIGNED(DATA_ALIGNED), .SEND_CLOCK(SEND_CLOCK), .SAP(1'b0));

/* Data Recovery Portion */

IBUFDS_LVDSEXT_25 LVDS_CDR_IN00(.O(DATAPAD[0]), .I(DATAINP[0]), .IB(DATAINN[0]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN01(.O(DATAPAD[1]), .I(DATAINP[1]), .IB(DATAINN[1]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN02(.O(DATAPAD[2]), .I(DATAINP[2]), .IB(DATAINN[2]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN03(.O(DATAPAD[3]), .I(DATAINP[3]), .IB(DATAINN[3]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN04(.O(DATAPAD[4]), .I(DATAINP[4]), .IB(DATAINN[4]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN05(.O(DATAPAD[5]), .I(DATAINP[5]), .IB(DATAINN[5]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN06(.O(DATAPAD[6]), .I(DATAINP[6]), .IB(DATAINN[6]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN07(.O(DATAPAD[7]), .I(DATAINP[7]), .IB(DATAINN[7]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN08(.O(DATAPAD[8]), .I(DATAINP[8]), .IB(DATAINN[8]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN09(.O(DATAPAD[9]), .I(DATAINP[9]), .IB(DATAINN[9]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN10(.O(DATAPAD[10]), .I(DATAINP[10]), .IB(DATAINN[10]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN11(.O(DATAPAD[11]), .I(DATAINP[11]), .IB(DATAINN[11]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN12(.O(DATAPAD[12]), .I(DATAINP[12]), .IB(DATAINN[12]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN13(.O(DATAPAD[13]), .I(DATAINP[13]), .IB(DATAINN[13]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN14(.O(DATAPAD[14]), .I(DATAINP[14]), .IB(DATAINN[14]));
IBUFDS_LVDSEXT_25 LVDS_CDR_IN15(.O(DATAPAD[15]), .I(DATAINP[15]), .IB(DATAINN[15]));

ISERDES ISERDES_CDR_IN00(.O(IDELAY2FDR[0]), .Q1(DI[3]), .Q2(DI[2]), .Q3(DI[1]), .Q4(DI[0]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[0]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN01(.O(IDELAY2FDR[1]), .Q1(DI[7]), .Q2(DI[6]), .Q3(DI[5]), .Q4(DI[4]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[1]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN02(.O(IDELAY2FDR[2]), .Q1(DI[11]), .Q2(DI[10]), .Q3(DI[9]), .Q4(DI[8]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[2]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN03(.O(IDELAY2FDR[3]), .Q1(DI[15]), .Q2(DI[14]), .Q3(DI[13]), .Q4(DI[12]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[3]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN04(.O(IDELAY2FDR[4]), .Q1(DI[19]), .Q2(DI[18]), .Q3(DI[17]), .Q4(DI[16]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[4]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN05(.O(IDELAY2FDR[5]), .Q1(DI[23]), .Q2(DI[22]), .Q3(DI[21]), .Q4(DI[20]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[5]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN06(.O(IDELAY2FDR[6]), .Q1(DI[27]), .Q2(DI[26]), .Q3(DI[25]), .Q4(DI[24]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[6]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN07(.O(IDELAY2FDR[7]), .Q1(DI[31]), .Q2(DI[30]), .Q3(DI[29]), .Q4(DI[28]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[7]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN08(.O(IDELAY2FDR[8]), .Q1(DI[35]), .Q2(DI[34]), .Q3(DI[33]), .Q4(DI[32]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[8]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN09(.O(IDELAY2FDR[9]), .Q1(DI[39]), .Q2(DI[38]), .Q3(DI[37]), .Q4(DI[36]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[9]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN10(.O(IDELAY2FDR[10]), .Q1(DI[43]), .Q2(DI[42]), .Q3(DI[41]), .Q4(DI[40]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[10]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN11(.O(IDELAY2FDR[11]), .Q1(DI[47]), .Q2(DI[46]), .Q3(DI[45]), .Q4(DI[44]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[11]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN12(.O(IDELAY2FDR[12]), .Q1(DI[51]), .Q2(DI[50]), .Q3(DI[49]), .Q4(DI[48]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[12]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN13(.O(IDELAY2FDR[13]), .Q1(DI[55]), .Q2(DI[54]), .Q3(DI[53]), .Q4(DI[52]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[13]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN14(.O(IDELAY2FDR[14]), .Q1(DI[59]), .Q2(DI[58]), .Q3(DI[57]), .Q4(DI[56]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[14]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));
ISERDES ISERDES_CDR_IN15(.O(IDELAY2FDR[15]), .Q1(DI[63]), .Q2(DI[62]), .Q3(DI[61]), .Q4(DI[60]), .Q5(), .Q6(), .SHIFTOUT1(), .SHIFTOUT2(), .BITSLIP(BITSLIP), .CE1(SCE), .CE2(), .CLK(RXCLK), .CLKDIV(RXCLKDIV), .D(DATAPAD[15]), .DLYCE(ICE), .DLYINC(INC), .DLYRST(IRST), .OCLK(), .REV(1'b0), .SHIFTIN1(), .SHIFTIN2(), .SR(IRST));


// synopsys translate_off
defparam ISERDES_CDR_IN00.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN01.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN02.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN03.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN04.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN05.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN06.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN07.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN08.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN09.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN10.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN11.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN12.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN13.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN14.BITSLIP_ENABLE = "TRUE";
defparam ISERDES_CDR_IN15.BITSLIP_ENABLE = "TRUE";

defparam ISERDES_CDR_IN00.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN01.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN02.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN03.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN04.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN05.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN06.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN07.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN08.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN09.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN10.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN11.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN12.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN13.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN14.DATA_RATE = "SDR";
defparam ISERDES_CDR_IN15.DATA_RATE = "SDR";

defparam ISERDES_CDR_IN00.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN01.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN02.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN03.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN04.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN05.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN06.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN07.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN08.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN09.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN10.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN11.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN12.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN13.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN14.DATA_WIDTH = 4;
defparam ISERDES_CDR_IN15.DATA_WIDTH = 4;

defparam ISERDES_CDR_IN00.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN01.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN02.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN03.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN04.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN05.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN06.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN07.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN08.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN09.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN10.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN11.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN12.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN13.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN14.INTERFACE_TYPE = "NETWORKING";
defparam ISERDES_CDR_IN15.INTERFACE_TYPE = "NETWORKING";

defparam ISERDES_CDR_IN00.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN01.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN02.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN03.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN04.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN05.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN06.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN07.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN08.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN09.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN10.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN11.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN12.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN13.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN14.IOBDELAY = "BOTH";
defparam ISERDES_CDR_IN15.IOBDELAY = "BOTH";

defparam ISERDES_CDR_IN00.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN01.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN02.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN03.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN04.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN05.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN06.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN07.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN08.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN09.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN10.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN11.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN12.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN13.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN14.IOBDELAY_TYPE = "VARIABLE";
defparam ISERDES_CDR_IN15.IOBDELAY_TYPE = "VARIABLE";

defparam ISERDES_CDR_IN00.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN01.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN02.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN03.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN04.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN05.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN06.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN07.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN08.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN09.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN10.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN11.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN12.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN13.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN14.IOBDELAY_VALUE = 0;
defparam ISERDES_CDR_IN15.IOBDELAY_VALUE = 0;

defparam ISERDES_CDR_IN00.NUM_CE = 1;
defparam ISERDES_CDR_IN01.NUM_CE = 1;
defparam ISERDES_CDR_IN02.NUM_CE = 1;
defparam ISERDES_CDR_IN03.NUM_CE = 1;
defparam ISERDES_CDR_IN04.NUM_CE = 1;
defparam ISERDES_CDR_IN05.NUM_CE = 1;
defparam ISERDES_CDR_IN06.NUM_CE = 1;
defparam ISERDES_CDR_IN07.NUM_CE = 1;
defparam ISERDES_CDR_IN08.NUM_CE = 1;
defparam ISERDES_CDR_IN09.NUM_CE = 1;
defparam ISERDES_CDR_IN10.NUM_CE = 1;
defparam ISERDES_CDR_IN11.NUM_CE = 1;
defparam ISERDES_CDR_IN12.NUM_CE = 1;
defparam ISERDES_CDR_IN13.NUM_CE = 1;
defparam ISERDES_CDR_IN14.NUM_CE = 1;
defparam ISERDES_CDR_IN15.NUM_CE = 1;

defparam ISERDES_CDR_IN00.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN01.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN02.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN03.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN04.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN05.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN06.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN07.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN08.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN09.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN10.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN11.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN12.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN13.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN14.SERDES_MODE = "MASTER";
defparam ISERDES_CDR_IN15.SERDES_MODE = "MASTER";
// synopsys translate_on

// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN00 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN01 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN02 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN03 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN04 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN05 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN06 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN07 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN08 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN09 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN10 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN11 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN12 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN13 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN14 is TRUE;
// synthesis attribute BITSLIP_ENABLE of ISERDES_CDR_IN15 is TRUE;

// synthesis attribute DATA_RATE of ISERDES_CDR_IN00 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN01 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN02 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN03 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN04 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN05 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN06 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN07 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN08 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN09 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN10 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN11 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN12 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN13 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN14 is SDR;
// synthesis attribute DATA_RATE of ISERDES_CDR_IN15 is SDR;

// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN00 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN01 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN02 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN03 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN04 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN05 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN06 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN07 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN08 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN09 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN10 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN11 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN12 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN13 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN14 is 4;
// synthesis attribute DATA_WIDTH of ISERDES_CDR_IN15 is 4;

// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN00 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN01 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN02 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN03 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN04 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN05 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN06 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN07 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN08 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN09 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN10 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN11 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN12 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN13 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN14 is NETWORKING;
// synthesis attribute INTERFACE_TYPE of ISERDES_CDR_IN15 is NETWORKING;
   
// synthesis attribute IOBDELAY of ISERDES_CDR_IN00 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN01 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN02 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN03 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN04 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN05 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN06 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN07 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN08 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN09 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN10 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN11 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN12 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN13 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN14 is IFD;
// synthesis attribute IOBDELAY of ISERDES_CDR_IN15 is IFD;

// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN00 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN01 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN02 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN03 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN04 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN05 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN06 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN07 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN08 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN09 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN10 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN11 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN12 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN13 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN14 is VARIABLE;
// synthesis attribute IOBDELAY_TYPE of ISERDES_CDR_IN15 is VARIABLE;

// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN00 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN01 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN02 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN03 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN04 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN05 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN06 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN07 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN08 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN09 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN10 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN11 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN12 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN13 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN14 is 0;
// synthesis attribute IOBDELAY_VALUE of ISERDES_CDR_IN15 is 0;

// synthesis attribute NUM_CE of ISERDES_CDR_IN00 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN01 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN02 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN03 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN04 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN05 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN06 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN07 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN08 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN09 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN10 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN11 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN12 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN13 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN14 is 1;
// synthesis attribute NUM_CE of ISERDES_CDR_IN15 is 1;

// synthesis attribute SERDES_MODE of ISERDES_CDR_IN00 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN01 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN02 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN03 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN04 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN05 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN06 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN07 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN08 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN09 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN10 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN11 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN12 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN13 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN14 is MASTER;
// synthesis attribute SERDES_MODE of ISERDES_CDR_IN15 is MASTER;


//FIFO Control Logic
FDR FIN_BUS_ALIGNED(.Q(FINAL_BUS_ALIGNED), .C(RXCLKDIV), .D(BUS_ALIGNED), .R(RST));

assign RDEN = ((((~AEMPTY) & FINAL_BUS_ALIGNED) | AFULL | FULL) & LOCKED);
assign WREN = (FINAL_BUS_ALIGNED &(~AFULL));

assign RDEN2 = ((((~AEMPTY2) & FINAL_BUS_ALIGNED) | AFULL2 | FULL2) & LOCKED);
assign WREN2 = (FINAL_BUS_ALIGNED &(~AFULL2));

//Data FIFO to move from RXCLKDIV domain to GCLKDIV domain
FIFO16 U_SDR_LVDS_TOP_FIFO160(.ALMOSTEMPTY(AEMPTY), .ALMOSTFULL(AFULL), .DO(DATA_OUT[31:0]), .DOP(), .EMPTY(), .FULL(FULL), .RDCOUNT(), .RDERR(), .WRCOUNT(), .WRERR(), .DI(DI[31:0]), .DIP(), .RDCLK(GCLKDIV), .RDEN(RDEN), .RST(IRST), .WRCLK(RXCLKDIV), .WREN(WREN));
FIFO16 U_SDR_LVDS_TOP_FIFO161(.ALMOSTEMPTY(AEMPTY2), .ALMOSTFULL(AFULL2), .DO(DATA_OUT[63:32]), .DOP(), .EMPTY(), .FULL(FULL2), .RDCOUNT(), .RDERR(), .WRCOUNT(), .WRERR(), .DI(DI[63:32]), .DIP(), .RDCLK(GCLKDIV), .RDEN(RDEN2),.RST(IRST), .WRCLK(RXCLKDIV), .WREN(WREN2));

// synopsys translate_off
defparam U_SDR_LVDS_TOP_FIFO160.ALMOST_FULL_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO160.ALMOST_EMPTY_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO160.DATA_WIDTH = 36;

defparam U_SDR_LVDS_TOP_FIFO161.ALMOST_FULL_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO161.ALMOST_EMPTY_OFFSET = 12'h032;
defparam U_SDR_LVDS_TOP_FIFO161.DATA_WIDTH = 36;	
// synopsys translate_on

// synthesis attribute ALMOST_FULL_OFFSET of U_SDR_LVDS_TOP_FIFO160 is 12'h032;
// synthesis attribute ALMOST_EMPTY_OFFSET of U_SDR_LVDS_TOP_FIFO160 is 12'h032;
// synthesis attribute DATA_WIDTH of U_SDR_LVDS_TOP_FIFO160 is 36;

// synthesis attribute ALMOST_FULL_OFFSET of U_SDR_LVDS_TOP_FIFO161 is 12'h032;
// synthesis attribute ALMOST_EMPTY_OFFSET of U_SDR_LVDS_TOP_FIFO161 is 12'h032;
// synthesis attribute DATA_WIDTH of U_SDR_LVDS_TOP_FIFO161 is 36;


endmodule

`timescale  1 ns / 10 ps
module count_to_16(clk, rst, count, counter_value);

//This module counts from 0 to 16

input clk, rst, count;
output [3:0] counter_value;

reg [3:0] counter_value_preserver/*synthesis syn_noprune = 1*/;

assign counter_value = (count) ? counter_value_preserver + 1 : 4'h0;

always@(posedge clk or posedge rst)
 if(rst == 1'b1)
   counter_value_preserver = 4'h0;
 else
   counter_value_preserver = counter_value;

endmodule


`timescale  1 ns / 10 ps
module iserdes_alignment_machine(RXCLKDIV, RST, SAMPLED_CLOCK, IRDY, USE_BITSLIP, RXDATA, TRAINING_PATTERN, INC, ICE, BITSLIP, DATA_ALIGNED, SEND_CLOCK, SAP);

/* Clock to Data Alignment Control Unit */

input		RXCLKDIV, RST, IRDY, USE_BITSLIP, SAP;
input	[3:0]	SAMPLED_CLOCK;
input	[3:0]	RXDATA, TRAINING_PATTERN;
output		INC, ICE, BITSLIP, DATA_ALIGNED, SEND_CLOCK;

reg		INC, ICE, BITSLIP, COUNT, UD, STORE, SEND_CLOCK;
reg	[3:0] 	CURRENT_STATE, NEXT_STATE;

wire	[6:0]	COUNT_VALUE, CVS;

count_to_128 machine_counter(.clk(RXCLKDIV), .rst(RST), .count(COUNT), .ud(UD), .counter_value(COUNT_VALUE));
seven_bit_reg_w_ce tap_reserve(.Q(CVS), .CLK(RXCLKDIV), .CE(STORE), .D(COUNT_VALUE), .RST(RST));  

FDR count_reg(.Q(DATA_ALIGNED), .C(RXCLKDIV), .D(DATA_ALIGNEDx), .R(RST));

assign DATA_ALIGNEDx = ((~CURRENT_STATE[3]) & CURRENT_STATE[2] & CURRENT_STATE[1] & (CURRENT_STATE[0]));
//assign DATA_ALIGNEDx = (NEXT_STATE[2] & NEXT_STATE[1] & (NEXT_STATE[0]));

always@(posedge RXCLKDIV or posedge RST)
begin
 if(RST == 1'b1)
  begin
   //Initial Values
   CURRENT_STATE = 4'b0000;
  end
 else
  begin
   //Transition Values
   CURRENT_STATE = NEXT_STATE;
  end
end

//State Machine to detect X10 or X010 pattern
always@(SAMPLED_CLOCK or IRDY or USE_BITSLIP or COUNT_VALUE or RXDATA or TRAINING_PATTERN or CURRENT_STATE or CVS or SAP)
 begin
  INC = 1'b0;
  ICE = 1'b0;
  COUNT = 1'b0;
  BITSLIP = 1'b0;
  UD = 1'b0;
  STORE = 1'b0;
  SEND_CLOCK = 1'b0;
  case(CURRENT_STATE)
   4'b0000: begin
    //Reset State
    SEND_CLOCK = 1'b1;
    if((IRDY == 1'b1) && (SAP == 1'b0))
    	NEXT_STATE = 4'b0001;
    else
    	NEXT_STATE = 4'b0000;
   end
   4'b0001: begin
    //Start of clock pattern detection
    SEND_CLOCK = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;    
    else
    if(SAMPLED_CLOCK == 4'b0000)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b0001;
     end
    else
    if(SAMPLED_CLOCK != 4'b0000)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b1011;
     end
   end
   
   4'b1011: begin
   INC = 1'b1;
   ICE = 1'b1;
   NEXT_STATE = 4'b0010;
   end
   
   4'b0010: begin
    SEND_CLOCK = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;    
    else
    if(SAMPLED_CLOCK == 4'b1111)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b0010;
     end
    else
    if(SAMPLED_CLOCK != 4'b1111)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	COUNT = 1'b1;
    	UD = 1'b1;
    	NEXT_STATE = 4'b1001;
     end
   end 
   
   4'b1001:begin
   INC = 1'b1;
   ICE = 1'b1;
   COUNT = 1'b1;
   UD = 1'b1;
   NEXT_STATE = 4'b0011;
   end
   
   4'b0011: begin
    //Start to determine data window width
    SEND_CLOCK = 1'b1;
    COUNT = 1'b1;
    UD = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;
    else
    if(SAMPLED_CLOCK == 4'b0000)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b0011;
     end
    else
    if(SAMPLED_CLOCK != 4'b0000)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b1010; //Changed
     end
   end
   
   4'b1010:begin
   INC = 1'b1;
   ICE = 1'b1;
   COUNT = 1'b1;
   UD = 1'b1;
   NEXT_STATE = 4'b1000;
   end
   
   
   4'b1000: begin
    //Determine 2nd Half of data window width
    SEND_CLOCK = 1'b1;
    COUNT = 1'b1;
    UD = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;
    else
    if(SAMPLED_CLOCK == 4'b1111)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	NEXT_STATE = 4'b1000;
     end
    else
    if(SAMPLED_CLOCK != 4'b1111)
     begin
     	INC = 1'b1;
    	ICE = 1'b1;
    	STORE = 1'b1;
    	NEXT_STATE = 4'b0100; //Changed
     end
   end
   
   
   4'b0100: begin
    //Subtract to middle of data window
    SEND_CLOCK = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;
    else
    if(COUNT_VALUE != {1'b0,CVS[6:1]})  
     begin
     	COUNT = 1'b1;
     	UD = 1'b0;
     	INC = 1'b0;
     	ICE = 1'b1;
     	NEXT_STATE = 4'b0100;
     end
    else
    if(COUNT_VALUE == {1'b0,CVS[6:1]})
     begin
     	COUNT = 1'b0;
     	UD = 1'b0;
     	INC = 1'b0;
     	ICE = 1'b1;
 	NEXT_STATE = 4'b0101;
     end
   end
   4'b0101: begin
    //Bitslip Portion
    COUNT = 1'b1;
    UD = 1'b1;
    if((IRDY == 1'b0) || (SAP == 1'b1))
    	NEXT_STATE = 4'b0000;    
    else
    if((USE_BITSLIP == 1'b0) || (RXDATA == TRAINING_PATTERN))
    	NEXT_STATE = 4'b0111;
    else
    if(COUNT_VALUE != 7'h0F)
    	NEXT_STATE = 4'b0101;
    else
    if(COUNT_VALUE == 7'h0F)
    	NEXT_STATE = 4'b0110;    
   end  
   4'b0110: begin
    if((IRDY == 1'b0) || (SAP == 1'b1))
        NEXT_STATE = 4'b0000;
    else
     begin
    	BITSLIP = 1'b1;
    	NEXT_STATE = 4'b0101;
     end
   end  
   4'b0111: begin
    //Finish
    if((IRDY == 1'b0) || (SAP == 1'b1))
        NEXT_STATE = 4'b0000;
    else
    	NEXT_STATE = 4'b0111;
   end  
   default: begin
    INC = 1'b0;
    ICE = 1'b0;
    COUNT = 1'b0;
    UD = 1'b0;
    BITSLIP = 1'b0;
    STORE = 1'b0;
    NEXT_STATE = 4'b0000;
   end
  endcase
 end


endmodule

`timescale  1 ns / 10 ps
module seven_bit_reg_w_ce(Q, CLK, CE, D, RST);

input	[6:0]	D;
input		CLK, CE, RST;

output	[6:0]	Q;	

FDRE bit0(.Q(Q[0]), .C(CLK), .CE(CE), .D(D[0]), .R(RST));
FDRE bit1(.Q(Q[1]), .C(CLK), .CE(CE), .D(D[1]), .R(RST));
FDRE bit2(.Q(Q[2]), .C(CLK), .CE(CE), .D(D[2]), .R(RST));
FDRE bit3(.Q(Q[3]), .C(CLK), .CE(CE), .D(D[3]), .R(RST));
FDRE bit4(.Q(Q[4]), .C(CLK), .CE(CE), .D(D[4]), .R(RST));
FDRE bit5(.Q(Q[5]), .C(CLK), .CE(CE), .D(D[5]), .R(RST));
FDRE bit6(.Q(Q[6]), .C(CLK), .CE(CE), .D(D[6]), .R(RST));

endmodule


`timescale  1 ns / 10 ps
module count_to_128(clk, rst, count, ud, counter_value);

//This module counts up/down between 0 to 128

input clk, rst, count, ud;
output [6:0] counter_value;

wire [6:0] counter_value_preserver;
reg [6:0] counter_value/*synthesis syn_noprune = 1*/;

always@(posedge clk or posedge rst)
begin
if(rst == 1'b1)
 counter_value = 7'h00;
else
 begin
  case({count,ud})
   2'b00: counter_value = 7'h00;
   2'b01: counter_value = 7'h00;
   2'b10: counter_value = counter_value_preserver - 1;
   2'b11: counter_value = counter_value_preserver + 1;
   default: counter_value = 7'h00;
  endcase
 end
end

assign counter_value_preserver = counter_value;

endmodule

`timescale  1 ns / 10 ps
module rst_machine                                                
        (                                                         
        CLK_generic,                                              
        RST_stimulus,                                             
        IRDY,                                                     
                                                                  
        DOMAIN_RST                                                
        );                                                        
                                                                  
input           CLK_generic;                                      
input           RST_stimulus;                                     
input           IRDY;                                             
                                                                  
output          DOMAIN_RST;                                       
                                                                  
reg             DOMAIN_RST;                                       
reg             COUNT;                                            
reg     [1:0]   CURRENT_STATE, NEXT_STATE;                        
                                                                  
wire    [3:0]   COUNT_VALUE;                                      
                                                                  
count_to_16 machine_counter(.clk(CLK_generic), .rst(RST_stimulus),
.count(COUNT), .counter_value(COUNT_VALUE));                      
                                                                  
always@(posedge CLK_generic or posedge RST_stimulus)              
begin                                                             
 if(RST_stimulus == 1'b1)                                         
  begin                                                           
   //Initial Values                                               
   CURRENT_STATE = 2'b00;                                         
  end                                                             
 else                                                             
  begin                                                           
   //Transition Values                                            
   CURRENT_STATE = NEXT_STATE;                                    
  end                                                             
end                                                               
                                                                  
always@(IRDY or COUNT_VALUE or CURRENT_STATE)                     
 begin                                                            
  case(CURRENT_STATE)                                             
   2'b00: begin                                                   
    //While DCM is not locked, remain in this state               
    DOMAIN_RST = 1'b0;                                            
    COUNT = 1'b0;                                                 
    if(IRDY == 1'b1)                                              
        NEXT_STATE = 2'b01;                                       
    else                                                          
        NEXT_STATE = 2'b00;                                       
   end                                                            
                                                                  
   2'b01: begin                                                   
    //Start of DOMAIN_RST and hold for 16 clock cycles.           
    DOMAIN_RST = 1'b1;                                            
    COUNT = 1'b1;                                                 
    if(IRDY == 1'b0)                                              
        NEXT_STATE = 2'b00;                                       
    else                                                          
    if(COUNT_VALUE != 4'hF)                                       
        NEXT_STATE = 2'b01;                                       
    else                                                          
        if(COUNT_VALUE == 4'hF)                                   
        NEXT_STATE = 2'b10;                                       
   end                                                            
                                                                  
   2'b10: begin                                                   
    DOMAIN_RST = 1'b0;                                            
    COUNT = 1'b0;                                                 
    if(IRDY == 1'b0)                                              
        NEXT_STATE = 2'b00;                                       
    else                                                          
        NEXT_STATE = 2'b10;                                       
   end                                                            
   default: begin                                                 
    DOMAIN_RST = 1'b0;                                            
    COUNT = 1'b0;                                                 
    NEXT_STATE = 2'b00;                                           
   end                                                            
  endcase                                                         
 end                                                              
                                                                  
endmodule